function model = cnnInitialize(param, dataSize, classLabel)

% get the parameter for initializing the network
network = param.network;
numLayer = length(network);

% initialize the model
model = [];
model.numLayer = numLayer;
model.layers = cell(numLayer-1, 1);

model.layers{1}.type = 'input';
model.layers{1}.layerSize = [dataSize, 1];
for l = 2 : numLayer
	if strcmp(network{l}.type , 'convolution')
		model.layers{l}.type = 'convolution';
		model.layers{l}.actFun = network{l}.actFun;
		
		preLayerSize = model.layers{l-1}.layerSize;
		thisMapSize = preLayerSize(1:2) - network{l}.kernelSize + 1;
		model.layers{l}.layerSize = [thisMapSize network{l}.outputMaps];

		model.layers{l}.weight = rand([network{l}.kernelSize, network{l}.kernelSize, model.layers{l-1}.layerSize(3), network{l}.outputMaps]);
		model.layers{l}.weight = (model.layers{l}.weight - 0.5) * 2 * sqrt( 6 / (network{l}.kernelSize.^2 * (network{l}.outputMaps + model.layers{l-1}.layerSize(3))));
		model.layers{l}.bias  = zeros(network{l}.outputMaps,1);
	elseif strcmp(network{l}.type , 'subsampling')
		model.layers{l}.type = 'subsampling';
		model.layers{l}.actFun = network{l}.actFun;
		model.layers{l}.scale = network{l}.scale;

		preLayerSize = model.layers{l-1}.layerSize;
		thisMapSize =  preLayerSize(1:2) ./ network{l}.scale;
		assert( all(thisMapSize == floor(thisMapSize)) , 'subsampling layer are not integers\n');
		model.layers{l}.layerSize = [ thisMapSize model.layers{l-1}.layerSize(3) ];

		model.layers{l}.weight = ones(model.layers{l-1}.layerSize(3), 1);
		model.layers{l}.bias  = zeros(model.layers{l-1}.layerSize(3), 1);
	elseif strcmp(network{l}.type , 'fullconnected')
		model.layers{l}.type = 'fullconnected';
		model.layers{l}.actFun = network{l}.actFun;
		model.layers{l}.layerSize = [network{l}.size 1];
		
		model.layers{l}.weight = rand(prod(model.layers{l}.layerSize), prod(model.layers{l-1}.layerSize));
		model.layers{l}.weight = (model.layers{l}.weight - 0.5) * 2 * sqrt( 6 / (prod(model.layers{l}.layerSize) + prod(model.layers{l-1}.layerSize)));
		model.layers{l}.bias = zeros(model.layers{l}.layerSize);
	else % softmax classification layer
		model.layers{l}.type = 'classification';
		model.layers{l}.actFun = network{l}.actFun;
		model.layers{l}.layerSize = [classLabel 1];

		model.layers{l}.weight = rand( prod(model.layers{l}.layerSize) , prod(model.layers{l-1}.layerSize));
		model.layers{l}.weight = (model.layers{l}.weight - 0.5) * 2 * sqrt( 6 / (prod(model.layers{l}.layerSize) + prod(model.layers{l-1}.layerSize)));
		%model.layers{l}.bias = zeros(prod(model.layers{l}.layerSize),1);
		%Softmax do not a have bias term
	end
end
