function model = cnnTrain(model, trainData, trainLabels, param)

opts = param.opts;
batchSize = opts.batchSize;
numExamples = size(trainData, ndims(trainData));
numBatches = ceil( numExamples ./ opts.batchSize );
numLayer = model.numLayer;

% initialize the running Gradient
runningGrad = cell(numLayer,1);
for l = 2 : numLayer
	if strcmp( model.layers{l}.type, 'classification')
		runningGrad{l}.weight = zeros(size(model.layers{l}.weight));
	else 
		runningGrad{l}.weight = zeros(size(model.layers{l}.weight));
		runningGrad{l}.bias = zeros(size(model.layers{l}.bias));
	end
end
% running cost for display
runningCost = 0;
tic;
for iter = 1 : opts.numEpochs
	fprintf('Training Epoch: %d / %d \n',iter,opts.numEpochs);
	shuffle = randperm(numExamples);
	for batch = 1 : numBatches
		fprintf('\t Batch: %d / %d \n', batch, numBatches);
		thisData = trainData(:,:,:,shuffle((batch-1)*batchSize+1 : min(batch * batchSize, numExamples)));
		thisLabel = trainLabels(:, shuffle((batch-1)*batchSize+1 : min(batch * batchSize, numExamples)));

		output = cnnFeedForward(model, thisData);
		[cost, grad] = cnnCost(model, output, thisLabel, param.model);
		[model, runningGrad] = applyGrad(model,grad,runningGrad, opts);
		if runningCost == 0
			runningCost = cost;
		else
			runningCost = 0.99 * runningCost + 0.01 * cost;
		end
	end
	fprintf('End of training Epoch: %d / %d \n',iter,opts.numEpochs);
	fprintf('Cost Value : %f\n', runningCost);
end
toc

save('model','model');