function main()

load trainData;
load trainLabels;

dataSize = size(squeeze(trainData(:,:,:,1)));
classLabels = size(trainLabels,1);

param = setParam();
model = cnnInitialize(param, dataSize, classLabels);

%% test the convlution and subsampling
%{
patchSize = 5;
for t = 1 : 1000
	image = randi([1 3]);
	feature = randi([1 6]);
	patchRow = randi([1 252]);
	patchCol = randi([1 252]);
	
	patch = trainData(patchRow : patchRow + patchSize -1 , patchCol : patchCol + patchSize - 1, :,image);
	patch = squeeze(patch);
	
	filter = model.layers{2}.weight(:,:,:,feature);
	filter = squeeze(filter);
	bias = model.layers{2}.bias(feature);
	
	groundTruth = filter(:)' * patch(:) + bias;
	groundTruth = 1 ./ ( 1 + exp(-groundTruth));
	my = output{2}.activation(patchRow, patchCol, image, feature);
	
	if abs(my - groundTruth) > 1e-9
		error('Convolution Error!\n');
	end

	for p = 1 : 100
		poolRow = randi([1 126]);
		poolCol = randi([1 126]);

		groundTruth = output{2}.activation(2*poolRow-1:2*poolRow, 2*poolCol-1:2*poolCol, image, feature);
		groundTruth = squeeze(groundTruth);
		groundTruth = mean(groundTruth(:));
		my = output{3}.activation(poolRow, poolCol, image, feature);

		if abs(my - groundTruth) > 1e-9
			error('Pooling Error!\n');
		end
	end
end

disp('passed Convolution and Pooling!');
%}

%% Test the gradient
%{
[cost grad] = cnnCost(model, output, trainLabels, param);

numGrad = computeNumericalGradient(@(x)testGradCost(x,trainData,trainLabels,param),model);

assert( norm(grad{2}.weight(:)-numGrad{2}.weight(:))./ norm(grad{2}.weight(:) + numGrad{2}.weight(:)) < 1e-9, 'BackPropagation is wrong \n');
%}

%% train the model
model = cnnTrain(model, trainData, trainLabels, param);


%% test the model
load testData;
load testLabels;

output = cnnFeedForward(model, testData);
[value, predictLabels] = max(output{model.numLayer}.activation,[],1);
[value, labels] = max(testLabels,[],1);

acc = mean(labels(:) == predictLabels(:));
fprintf('Accuracy: %0.3f%%\n', acc * 100);


